package fb.rt;
/** Copyright (c)2008-2010 Holobloc Inc. All rights reserved. */


import fb.rt.math.FBStatus;

/**
 * @author JHC
 * @version 20101116/JHC - Added testGetECState().
 * @version 20101023/JHC - Added testToString() to test for presence of State
 *          name.
 * @version 2010824/JHC - Updated testInitialize() for case when a component or
 *          an HMI fixture rejects the Resource.
 * @version 2010813/JHC - Updated testInitialize for case when fixture is a
 *          Device.
 * @version 2010809/JHC - Added testKill(), testReset(); updated testStart(),
 *          testStop(), testInitialize().
 * @version 2010808/JHC - Updated testStart(), testStop() to check state.
 * @version 20100524/JHC - Updated testConnectOV() for new implementation in
 *          AbstractAdapter. Moved testConnectOVxxx() methods to
 *          AbstractAdapterTestCase.
 * @version 20100405/JHC - Deleted forceGC() - No longer used.
 * @version 20090428/JHC - Restored testConnectOVInvalidOp(),
 *          testConnectOVNoExceptionInvalidOp() - Now supported again by
 *          simplifed connectOV() framework in FBInstance.
 * @version 20090427/JHC - Deleted testConnectOVInvalidOp(),
 *          testConnectOVNoExceptionInvalidOp() - abandoned this exception for
 *          consistency with exceptions in FBInstance.connectOV().
 *          testConnectOV(), testConnectOVNoException() - now not required by
 *          abstract method in superclass.
 * @version 20090425/JHC - Updated testConnectOV_0.
 * @version 20090328/JHC - Added String values of FBStatus constants.
 * @version 20090323/JHC - Added testConnectOV_0().
 * @version 20090321/JHC - Added testConnectOVInvalidOp(String, ANY),
 *          testConnectOVNoExceptionInvalidOp(String, ANY).
 * @version 20090129/JHC - Changed testConnectOV(String, boolean),
 *          testConnectOVNoException(String, boolean) to (String, ANY) - OV now
 *          treated same as IV.
 * @version 20090122/JHC - Added forceGC().
 * @version 20081231/JHC - Copied, updated from FBRT_CLDC.
 */
public abstract class FBInstanceTestCase extends AbstractAdapterTestCase {
	public static final String OK = String.valueOf(FBStatus.OK);

	public static final String INHIBITED = String.valueOf(FBStatus.INHIBITED);

	public static final String TYPE_CONVERSION_ERROR = String.valueOf(FBStatus.TYPE_CONVERSION_ERROR);

	public static final String DIVISION_BY_0 = String.valueOf(FBStatus.DIVISION_BY_0);

	public static final String OVERFLOW = String.valueOf(FBStatus.OVERFLOW);

	public static final String SELECTOR_OUT_OF_RANGE = String.valueOf(FBStatus.SELECTOR_OUT_OF_RANGE);

	public static final String INVALID_CHAR_POSITION = String.valueOf(FBStatus.INVALID_CHAR_POSITION);

	public static final String STRING_TOO_LONG = String.valueOf(FBStatus.STRING_TOO_LONG);

	public static final String INVALID_SFC_DIVERGE = String.valueOf(FBStatus.INVALID_SFC_DIVERGE);

	public static final String ACTION_CONTROL_CONTENTION = String.valueOf(FBStatus.ACTION_CONTROL_CONTENTION);

	public static final String FUNCTION_FAILURE = String.valueOf(FBStatus.FUNCTION_FAILURE);

	public static final String ITERATION_FAILURE = String.valueOf(FBStatus.ITERATION_FAILURE);

	public static final String ARRAY_SIZE_ERROR = String.valueOf(FBStatus.ARRAY_SIZE_ERROR);

	public static final String INVALID_SUBSCRIPT = String.valueOf(FBStatus.INVALID_SUBSCRIPT);

	/**
	 * Checks the behavior of <code>initialize(String, Resource)</code>. May be
	 * supplemented by subclasses.
	 */
	@Override
	public void testInitialize() throws Exception {
		FBInstance instance = ((FBInstance) getFixture());
		try {
			instance.initialize(null, null);
		} catch (FBRManagementException e) {
			// In case a component FB throws an exception
			assertEquals("INVALID_OPERATION", e.getMessage());
		}
		assertEquals("", instance.getName());
		assertNull(instance.getResource());
		if (instance instanceof Device)
			assertTrue(instance == instance.getDevice());
		else
			assertNull(instance.getDevice());
		try {
			instance.initialize("TEST", null);
		} catch (FBRManagementException e) {
			// In case a component FB throws an exception
			assertEquals("INVALID_OPERATION", e.getMessage());
		}
		assertEquals("TEST", instance.getName());
		assertNull(instance.getResource());
		if (instance instanceof Device)
			assertTrue(instance == instance.getDevice());
		else
			assertNull(instance.getDevice());
		try {
			instance.initialize(null, null);
		} catch (FBRManagementException e) {
			// In case a component FB throws an exception
			assertEquals("INVALID_OPERATION", e.getMessage());
		}
		assertEquals("TEST", instance.getName());
		assertNull(instance.getResource());
		if (instance instanceof Device)
			assertTrue(instance == instance.getDevice());
		else
			assertNull(instance.getDevice());
		Device device = new RMT_DEV("");
		Resource resource = (Resource) device.getManager().fbNamed("MGR");
		try {
			instance.initialize("TEST2", resource);
			assertTrue(instance.getResource() == resource);
			if (instance instanceof Device)
				assertTrue(instance == instance.getDevice());
			else
				assertTrue(instance.getDevice() == device);
		} catch (FBRManagementException e) {
			// In case a component FB or an HMI fixture throws an exception
			assertEquals("INVALID_OPERATION", e.getMessage());
		}
		assertEquals("TEST2", instance.getName());
	}

	/**
	 * Tests the start() method. May be supplemented by subclasses.
	 * @throws Exception The test failed.
	 */
	public void testStart() throws Exception {
		FBInstance fb = (FBInstance) getFixture();
		assertEquals(FB_STATE.IDLE, fb.getState());
		// IDLE to RUNNING
		fb.start();
		assertEquals(FB_STATE.RUNNING, fb.getState());
		// RUNNING to RUNNING
		fb.start();
		assertEquals(FB_STATE.RUNNING, fb.getState());
		// STOPPED to RUNNING
		fb.stop();
		assertEquals(FB_STATE.STOPPED, fb.getState());
		fb.start();
		assertEquals(FB_STATE.RUNNING, fb.getState());
		// KILLED to KILLED
		fb.kill();
		assertEquals(FB_STATE.KILLED, fb.getState());
		fb.start();
		assertEquals(FB_STATE.KILLED, fb.getState());
		// Leave in RUNNING state
		fb.reset();
		fb.start();
	}

	/**
	 * Tests the stop() method. May be supplemented by subclasses.
	 * @throws Exception The test failed.
	 */
	public void testStop() throws Exception {
		FBInstance fb = (FBInstance) getFixture();
		// IDLE to IDLE
		assertEquals(FB_STATE.IDLE, fb.getState());
		fb.stop();
		assertEquals(FB_STATE.IDLE, fb.getState());
		// RUNNING to STOPPED
		fb.start();
		assertEquals(FB_STATE.RUNNING, fb.getState());
		fb.stop();
		assertEquals(FB_STATE.STOPPED, fb.getState());
		// STOPPED to STOPPED
		fb.stop();
		assertEquals(FB_STATE.STOPPED, fb.getState());
		// KILLED to KILLED
		fb.start();
		assertEquals(FB_STATE.RUNNING, fb.getState());
		fb.kill();
		assertEquals(FB_STATE.KILLED, fb.getState());
		fb.stop();
		assertEquals(FB_STATE.KILLED, fb.getState());
		// Leave in STOPPED state
		fb.reset();
		fb.start();
		fb.stop();
	}

	/**
	 * Tests the kill() method. May be supplemented by subclasses.
	 * @throws Exception The test failed.
	 */
	public void testKill() throws Exception {
		FBInstance fb = (FBInstance) getFixture();
		// IDLE to IDLE
		assertEquals(FB_STATE.IDLE, fb.getState());
		fb.kill();
		assertEquals(FB_STATE.IDLE, fb.getState());
		// RUNNING to KILLED
		fb.start();
		assertEquals(FB_STATE.RUNNING, fb.getState());
		fb.kill();
		assertEquals(FB_STATE.KILLED, fb.getState());
		// KILLED to KILLED
		fb.kill();
		assertEquals(FB_STATE.KILLED, fb.getState());
		// STOPPED to STOPPED
		fb.reset();
		fb.start();
		fb.stop();
		assertEquals(FB_STATE.STOPPED, fb.getState());
		fb.kill();
		assertEquals(FB_STATE.STOPPED, fb.getState());
		// Leave with state = KILLED
		fb.reset();
		fb.start();
		fb.kill();
	}

	/**
	 * This test is only used for a concrete subclass of FBInstance that does
	 * not implement serviceEvent() - test is done in testEvents() in that case.
	 */
	@SuppressWarnings({ "rawtypes", "unchecked" })
	public final void testServiceEvent() throws Exception {
		Class[] parameterClasses = { EventServer.class };
		FBInstance fbInstance = (FBInstance) getFixture();
		for (Class aClass = ((FBInstance) getFixture()).getClass(); !aClass.equals(FBInstance.class); aClass = aClass
				.getSuperclass()) {
			try {
				aClass.getDeclaredMethod("serviceEvent", parameterClasses);
				return;
			} catch (NoSuchMethodException e) {
				continue;
			}
		}
		// serviceEvent() not found in subclasses, exercise here
		fbInstance.serviceEvent(fbInstance);
	}

	/**
	 * Tests the reset() method. May be supplemented by subclasses.
	 * @throws Exception The test failed.
	 */
	public void testReset() throws Exception {
		FBInstance fb = (FBInstance) getFixture();
		// IDLE to IDLE
		assertEquals(FB_STATE.IDLE, fb.getState());
		fb.reset();
		assertEquals(FB_STATE.IDLE, fb.getState());
		// RUNNING to RUNNING
		fb.start();
		assertEquals(FB_STATE.RUNNING, fb.getState());
		fb.reset();
		assertEquals(FB_STATE.RUNNING, fb.getState());
		// KILLED to IDLE
		fb.kill();
		fb.reset();
		assertEquals(FB_STATE.IDLE, fb.getState());
		// STOPPED to IDLE
		fb.start();
		fb.stop();
		assertEquals(FB_STATE.STOPPED, fb.getState());
		fb.reset();
		assertEquals(FB_STATE.IDLE, fb.getState());
	}

	/**
	 * Tests the <code>toString()</code> method. By default, tests that the
	 * result is in the <code>"Name : Type [State]"</code> format. Subclasses
	 * may over-ride if their <code>toString()</code> method returns some other
	 * value.
	 */
	@Override
	public void testToString() throws Exception {
		FB_STATE state = ((FBInstance) getFixture()).state;
		assertEquals(
				getFixture().getName() + " : " + getFixture().getType() + " [" + state.getValues()[state.value] + ']',
				getFixture().toString());
	}

	public void testGetECState() {
		FBInstance fb = (FBInstance) getFixture();
		assertEquals(fb.eccState, fb.getECState());
	}
}